package gov.anl.aps.dm.common.exceptions;

import gov.anl.aps.dm.common.constants.DmStatus;

/**
 * Generic DM exception, used as base class for all DM exceptions.
 */
public class DmException extends Exception {

    /**
     * Exception keys.
     */
    public static final String SIGNATURE_KEY = "__dm_exception__";
    public static final String TYPE_KEY = "type";
    public static final String CODE_KEY = "code";
    public static final String ARGS_KEY = "args";

    private String error = null;

    /**
     * Default constructor.
     */
    public DmException() {
        super();
    }

    /**
     * Constructor using error message.
     *
     * @param message error message
     */
    public DmException(String message) {
        super(message);
    }

    /**
     * Constructor using throwable object.
     *
     * @param throwable throwable object
     */
    public DmException(Throwable throwable) {
        super(throwable);
    }

    /**
     * Constructor using error message and throwable object.
     *
     * @param message error message
     * @param throwable throwable object
     */
    public DmException(String message, Throwable throwable) {
        super(message, throwable);
    }

    public int getErrorCode() {
        return DmStatus.DM_ERROR;
    }

    public void setErrorMessage(String error) {
        this.error = error;
    }

    public String getErrorMessage() {
        if (error != null) {
            return error;
        }
        return super.getMessage();
    }

    /**
     * Convert exception to string, overriding string output if error message is
     * set.
     *
     * @return exception string
     */
    @Override
    public String toString() {
        if (error != null) {
            return error;
        } else {
            return super.toString();
        }
    }
}
