/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.anl.aps.dm.portal.utilities;

import gov.anl.aps.dm.portal.model.entities.Experiment;
import gov.anl.aps.dm.portal.model.entities.ExperimentStation;
import gov.anl.aps.dm.portal.model.entities.UserInfo;
import gov.anl.aps.dm.portal.model.entities.UserSystemRole;

/**
 *
 * @author sveseli
 */
public class AuthorizationUtility {
    public static final int ADMINISTRATOR_SYSTEM_ROLE_ID = 1;
    public static final int MANAGER_SYSTEM_ROLE_ID = 2;
   
    public static final int PI_EXPERIMENT_ROLE_ID = 1;
    public static final int USER_EXPERIMENT_ROLE_ID = 2;
    
    public static boolean isAdministrator(UserInfo user) {
        return user.getUserSystemRoleList().stream().anyMatch((userSystemRole) -> (userSystemRole.getSystemRoleType().getId() == ADMINISTRATOR_SYSTEM_ROLE_ID));
    }
    
    public static boolean isStationManager(UserInfo user, ExperimentStation experimentStation) {
        return user.getUserSystemRoleList().stream().anyMatch((userSystemRole) -> (userSystemRole.getSystemRoleType().getId() == MANAGER_SYSTEM_ROLE_ID 
                && userSystemRole.getExperimentStation().getId().equals(experimentStation.getId())));
    }

    public static boolean isExperimentPi(UserInfo user, Experiment experiment) {
        return user.getUserExperimentRoleList().stream().anyMatch((userExperimentRole) -> (userExperimentRole.getExperimentRoleType().getId() == PI_EXPERIMENT_ROLE_ID 
                && userExperimentRole.getExperiment().getId().equals(experiment.getId())));
    }    

    public static boolean isExperimentUser(UserInfo user, Experiment experiment) {
        return user.getUserExperimentRoleList().stream().anyMatch((userExperimentRole) -> (userExperimentRole.getExperimentRoleType().getId() == USER_EXPERIMENT_ROLE_ID 
                && userExperimentRole.getExperiment().getId().equals(experiment.getId())));
    } 
    
    public static boolean canManageStation(UserInfo user, ExperimentStation experimentStation) {
        return isAdministrator(user) || isStationManager(user, experimentStation);
    }
    
    public static boolean canManageExperiment(UserInfo user, Experiment experiment) {
        return canManageStation(user, experiment.getExperimentStation()) || isExperimentPi(user, experiment);
    } 
    
    public static boolean canCreateExperiment(UserInfo user) {
        return !user.getUserSystemRoleList().isEmpty();
    }    
}
