#!/usr/bin/env python

from dm.common.utility.singleton import Singleton
from dm.common.utility.loggingManager import LoggingManager
from dm.common.utility.configurationManager import ConfigurationManager

class DmApiFactory(Singleton):


    def __init__(self):
        self.__configure()

    def __configure(self):
        self.logger = LoggingManager.getInstance().getLogger(self.__class__.__name__)
        configManager = ConfigurationManager.getInstance()
        (self.username,self.password) = configManager.parseLoginFile()
        self.protocol = configManager.getWebServiceProtocol()
        self.dsHost = configManager.getDsWebServiceHost()
        self.dsPort = configManager.getDsWebServicePort()
        self.daqHost = configManager.getDaqWebServiceHost()
        self.daqPort = configManager.getDaqWebServicePort()

    def getUserDsApi(self):
        from dm.ds_web_service.api.userDsApi import UserDsApi
        api = UserDsApi(self.username, self.password, self.dsHost, self.dsPort, self.protocol)
        return api

    def getExperimentDsApi(self):
        from dm.ds_web_service.api.experimentDsApi import ExperimentDsApi
        api = ExperimentDsApi(self.username, self.password, self.dsHost, self.dsPort, self.protocol)
        return api

    def getExperimentDaqApi(self):
        from dm.daq_web_service.api.experimentDaqApi import ExperimentDaqApi
        api = ExperimentDaqApi(self.username, self.password, self.daqHost, self.daqPort, self.protocol)
        return api

    def getBeamlineProposalApi(self):
        from dm.aps_bss.api.apsBssApi import ApsBssApi
        api = ApsBssApi()
        return api

    def getApsUserDbApi(self):
        from dm.aps_user_db.api.apsUserDbApi import ApsUserDbApi
        api = ApsUserDbApi()
        return api


####################################################################
# Testing

if __name__ == '__main__':
    pass

