#!/usr/bin/env python

from PyQt4.QtGui import QGridLayout, QSpacerItem, QSizePolicy, QPushButton, QWidget, \
    QFont, QLabel, QTableWidget, QTableWidgetItem, QColor, QAbstractItemView
from PyQt4.QtCore import Qt
from dmApiFactory import DmApiFactory
from objects.userInfo import UserInfo
 
# Define the experiments tab content:
class ExperimentsTab(QWidget):
    def __init__(self, stationName, parent, id=-1):
        super(ExperimentsTab, self).__init__(parent)
        self.stationName = stationName
        self.parent = parent
        self.experimentDsApi = DmApiFactory.getInstance().getExperimentDsApi()
        self.userApi = DmApiFactory.getInstance().getUserDsApi()
        self.experimentsTabLayout()

    # Sets up the tab's layout, each block is a row
    def experimentsTabLayout(self):
        grid = QGridLayout()

        labelFont = QFont('Arial', 18, QFont.Bold)
        lbl = QLabel(self.stationName + ' DM Experiment List', self)
        lbl.setAlignment(Qt.AlignCenter)
        lbl.setFont(labelFont)
        grid.addWidget(lbl, 0, 0, 1, 5)

        backBtn = QPushButton('Back', self)
        backBtn.clicked.connect(lambda: self.parent.setTab(0))
        backBtn.setMinimumWidth(100)
        grid.addWidget(backBtn, 1, 4, Qt.AlignCenter)

        grid.addItem(QSpacerItem(20, 30, QSizePolicy.Expanding), 2, 0)

        self.tableWidget = QTableWidget()
        self.updateList()
        grid.addWidget(self.tableWidget, 3, 0, 1, 5)

        grid.addItem(QSpacerItem(40, 20, QSizePolicy.Expanding), 4, 0)
        addBtn = QPushButton('Add Experiment', self)
        addBtn.clicked.connect(lambda: self.parent.setTab(4))
        addBtn.setMinimumWidth(150)
        addBtn.setFixedHeight(40)
        grid.addWidget(addBtn, 4, 1)
        grid.addItem(QSpacerItem(40, 20, QSizePolicy.Expanding), 4, 2)
        modBtn = QPushButton('Use Selected', self)
        modBtn.clicked.connect(self.setExperiment)
        modBtn.setMinimumWidth(150)
        modBtn.setFixedHeight(40)
        grid.addWidget(modBtn, 4, 3)
        grid.addItem(QSpacerItem(40, 20, QSizePolicy.Expanding), 4, 4)

        grid.addItem(QSpacerItem(20, 10, QSizePolicy.Expanding), 5, 0)

        self.experimentsTabWidget = QWidget()
        self.experimentsTabWidget.setLayout(grid)

    # Sets up the table structure and fills it with data
    def updateList(self):
        self.experimentList = self.experimentDsApi.getExperimentsByStation(self.stationName)
        self.tableWidget.setRowCount(len(self.experimentList))
        self.tableWidget.setColumnCount(3)
        self.tableWidget.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.tableWidget.setHorizontalHeaderLabels('Name;Description;Start Date;'.split(';'))
        self.tableWidget.cellDoubleClicked.connect(self.expandRow)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)

        i = 0
        for experiment in self.experimentList:
            rowName = QTableWidgetItem(experiment.get('name'))
            rowName.setData(Qt.UserRole, experiment.get('id'))
            rowDescription = QTableWidgetItem(experiment.get('description', ''))
            rowStartDate = QTableWidgetItem(str(experiment.get('startDate', '')))
            self.tableWidget.setItem(i, 0, rowName)
            self.tableWidget.setItem(i, 1, rowDescription)
            self.tableWidget.setItem(i, 2, rowStartDate)
            if (i % 2) == 0:
                rowName.setBackground(QColor.fromRgb(230, 241, 245))
                rowDescription.setBackground(QColor.fromRgb(230, 241, 245))
                rowStartDate.setBackground(QColor.fromRgb(230, 241, 245))
            i += 1


    # Expands the given row to fit the size of its contents
    def expandRow(self, row, column):
        self.tableWidget.resizeRowToContents(row)

    def setExperiment(self):
        id = self.tableWidget.item(self.tableWidget.currentRow(), 0).data(Qt.UserRole).toInt()[0]
        experiment = self.experimentDsApi.getExperimentById(id)
        self.parent.generalSettings = experiment.data
        userList = [self.userApi.getUserByUsername(user) for user in experiment['experimentUsernameList']]
        self.parent.currentUsers = [UserInfo(x['badge'], x['firstName'], x['id'],  x['lastName'], x['email']) for x in userList]
        self.parent.setTab(6)


        # PROBLEM reading self.dbPasswordFile in apsUserDbManager....I need this because I need to be able to get a user from a badge number
        # Is it true/okay that this will only work for argonne users due to the badge methodology?

