#!/usr/bin/env python

from PyQt4.QtGui import QGridLayout, QSpacerItem, QSizePolicy, QPushButton, QWidget, \
    QFont, QLabel, QTableWidget, QTableWidgetItem, QColor, QAbstractItemView, QLineEdit, QVBoxLayout, QHBoxLayout
from PyQt4.QtCore import Qt
from dmApiFactory import DmApiFactory


# Define the experiments tab content:
class GenParamsTab(QWidget):
    def __init__(self, stationName, parent, id=-1):
        super(GenParamsTab, self).__init__(parent)
        self.stationName = stationName
        self.parent = parent
        self.experimentDsApi = DmApiFactory.getInstance().getExperimentDsApi()
        self.genParamsTabLayout()

    # Sets up the tab's layout, each block is a row
    def genParamsTabLayout(self):
        grid = QGridLayout()

        labelFont = QFont('Arial', 18, QFont.Bold)
        lbl = QLabel(self.stationName + ' General Settings', self)
        lbl.setAlignment(Qt.AlignCenter)
        lbl.setFont(labelFont)
        grid.addWidget(lbl, 0, 0, 1, 4)

        backBtn = QPushButton('Back', self)
        backBtn.clicked.connect(lambda: self.parent.setTab(1))
        backBtn.setMinimumWidth(100)
        grid.addWidget(backBtn, 1, 1, Qt.AlignRight)


        self.nameField = QLineEdit()
        self.nameField.setPlaceholderText('Name')

        self.startDateField = QLineEdit()
        self.startDateField.setPlaceholderText('Start Date')

        self.endDateField = QLineEdit()
        self.endDateField.setPlaceholderText('End Date')

        self.typeField = QLineEdit()
        self.typeField.setPlaceholderText('Type')

        self.descField = QLineEdit()
        self.descField.setPlaceholderText('Description')

        self.currentUserTable = QTableWidget()

        toUserBtn = QPushButton('Modify Users', self)
        toUserBtn.clicked.connect(lambda: self.parent.setTab(5))
        toUserBtn.setMinimumWidth(100)
        toUserBtn.setMaximumWidth(150)

        saveSettingsBtn = QPushButton('Save', self)
        saveSettingsBtn.clicked.connect(self.updateParams)
        saveSettingsBtn.setMinimumWidth(100)
        saveSettingsBtn.setMaximumWidth(150)

        startDaqBtn = QPushButton('Start DAQ', self)
        startDaqBtn.clicked.connect(lambda: self.parent.setTab(0))
        startDaqBtn.setMinimumWidth(150)
        startDaqBtn.setMaximumWidth(200)
        startDaqBtn.setFixedHeight(40)

        startUploadBtn = QPushButton('Start Upload', self)
        startUploadBtn.clicked.connect(lambda: self.parent.setTab(0))
        startUploadBtn.setMinimumWidth(150)
        startUploadBtn.setMaximumWidth(200)
        startUploadBtn.setFixedHeight(40)

        hRow1 = QHBoxLayout()
        hRow1.addWidget(self.startDateField)
        hRow1.addWidget(self.endDateField)

        vColumn1 = QVBoxLayout()
        vColumn1.addWidget(self.nameField)
        vColumn1.addLayout(hRow1)
        vColumn1.addWidget(self.typeField)
        vColumn1.addWidget(self.descField)
        grid.addLayout(vColumn1, 2, 0)

        vColumn2 = QVBoxLayout()
        vColumn2.addWidget(self.currentUserTable)

        grid.addLayout(vColumn2, 2, 1)

        vColumn3 = QVBoxLayout()
        vColumn3.addWidget(saveSettingsBtn)
        vColumn3.setAlignment(saveSettingsBtn, Qt.AlignCenter)
        vColumn3.addWidget(startDaqBtn)
        vColumn3.setAlignment(startDaqBtn, Qt.AlignCenter)
        grid.addLayout(vColumn3, 3, 0)

        vColumn4 = QVBoxLayout()
        vColumn4.addWidget(toUserBtn)
        vColumn4.setAlignment(toUserBtn, Qt.AlignCenter)
        vColumn4.addWidget(startUploadBtn)
        vColumn4.setAlignment(startUploadBtn, Qt.AlignCenter)
        grid.addLayout(vColumn4, 3, 1)

        self.genParamsTabWidget = QWidget()
        self.genParamsTabWidget.setLayout(grid)

    # Fills ui with any known information
    def fillParams(self):
        for key in self.parent.generalSettings:
            if key == 'name':
                self.nameField.setText(self.parent.generalSettings[key])
            elif key == 'date':
                self.datefield.setText(self.parent.generalSettings[key])
            elif key == 'type':
                self.typeField.setText(self.parent.generalSettings[key])
            elif key == 'description':
                self.descField.setText(self.parent.generalSettings[key])


        # Set up the current users table
        self.currentUserTable.setRowCount(len(self.parent.currentUsers))
        self.currentUserTable.setColumnCount(4)
        self.currentUserTable.setSelectionBehavior(QAbstractItemView.SelectRows)
        #for i, size in enumerate(self.colWidths):
        #    self.currentUserTable.horizontalHeader().resizeSection(i, size)
        self.currentUserTable.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.currentUserTable.setHorizontalHeaderLabels('Badge;First;Last;Email;'.split(';'))
        self.currentUserTable.cellDoubleClicked.connect(lambda row, column: self.expandRow(row, column, self.currentUserTable))

        i = 0
        for experimenter in self.parent.currentUsers:
            rowBadge = QTableWidgetItem(experimenter.getBadge())
            rowBadge.setData(Qt.UserRole, experimenter.getID())
            rowFirstName = QTableWidgetItem(experimenter.getFirstName())
            rowLastName = QTableWidgetItem(experimenter.getLastName())
            try:
                rowEmail = QTableWidgetItem(experimenter.getEmail())
            except KeyError:
                rowEmail = QTableWidgetItem('')
            self.currentUserTable.setItem(i, 0, rowBadge)
            self.currentUserTable.setItem(i, 1, rowFirstName)
            self.currentUserTable.setItem(i, 2, rowLastName)
            self.currentUserTable.setItem(i, 3, rowEmail)
            if (i % 2) == 0:
                rowBadge.setBackground(QColor.fromRgb(230, 241, 245))
                rowFirstName.setBackground(QColor.fromRgb(230, 241, 245))
                rowLastName.setBackground(QColor.fromRgb(230, 241, 245))
                rowEmail.setBackground(QColor.fromRgb(230, 241, 245))
            i += 1

        self.currentUserTable.horizontalHeader().setStretchLastSection(True)

    # Updates ui with information supplied by signal
    def updateParams(self):
        self.parent.generalSettings['name'] = self.nameField.text()
        self.parent.generalSettings['date'] = self.dateField.text()
        self.parent.generalSettings['type'] = self.typeField.text()
        self.parent.generalSettings['description'] = self.descField.text()
