#!/usr/bin/env python

from PyQt4.QtGui import QGridLayout, QSpacerItem, QSizePolicy, QPushButton, QWidget, \
    QFont, QLabel, QTableWidget, QTableWidgetItem, QColor, QAbstractItemView, QComboBox, QVBoxLayout, QHBoxLayout, \
    QFontMetrics, QTableView, QStandardItemModel, QStandardItem
from PyQt4.QtCore import Qt
from dmApiFactory import DmApiFactory
from objects.userInfo import UserInfo


# Define the experiments tab content:
class ManageUsersTab(QWidget):
    def __init__(self, stationName, parent, id=-1):
        super(ManageUsersTab, self).__init__(parent)
        self.stationName = stationName
        self.parent = parent
        self.experimentPropApi = DmApiFactory.getInstance().getBeamlineProposalApi()
        self.userApi = DmApiFactory.getInstance().getUserDsApi()
        self.manageUsersTabLayout()

    # Sets up the tab's layout, each block is a row
    def manageUsersTabLayout(self):
        grid = QGridLayout()

        labelFont = QFont('Arial', 18, QFont.Bold)
        lbl = QLabel(self.stationName + ' User Management', self)
        lbl.setAlignment(Qt.AlignCenter)
        lbl.setFont(labelFont)
        grid.addWidget(lbl, 0, 0, 1, 3)

        backBtn = QPushButton('Back', self)
        backBtn.clicked.connect(lambda: self.parent.setTab(6))
        backBtn.setMinimumWidth(100)
        grid.addWidget(backBtn, 1, 2, Qt.AlignRight)

        self.currentUserTable = QTableWidget()
        self.availableUserTable = QTableView()
        self.availableUserTableModel = QStandardItemModel()
        self.availableUserTable.setModel(self.availableUserTableModel)
        moveUserBtn = QPushButton('<->', self)
        moveUserBtn.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)

        currentLabel = QLabel('Current Users', self)
        currentLabel.setAlignment(Qt.AlignCenter)
        currentLabel.setFixedHeight(30)
        self.userDropdown = QComboBox(self)
        self.userDropdown.currentIndexChanged.connect(self.updateUsers)
        self.userDropdown.setMaximumWidth(250)
        self.updateDropdown()

        saveUsersBtn = QPushButton('Save and Return', self)
        saveUsersBtn.clicked.connect(lambda: self.parent.setTab(0))
        saveUsersBtn.setFixedHeight(40)

        grid.addItem(QSpacerItem(20, 10, QSizePolicy.Expanding, QSizePolicy.Minimum), 6, 0)

        hRow1 = QHBoxLayout()
        hRow1.addWidget(self.currentUserTable)
        hRow1.addWidget(moveUserBtn)
        hRow1.addWidget(self.availableUserTable)

        vColumn1 = QVBoxLayout()
        vColumn1.addWidget(currentLabel)
        vColumn1.addWidget(self.currentUserTable)
        grid.addLayout(vColumn1, 2, 0)

        vColumn2 = QVBoxLayout()
        vColumn2.addWidget(moveUserBtn)
        grid.addLayout(vColumn2, 2, 1)

        vColumn3 = QVBoxLayout()
        vColumn3.addWidget(self.userDropdown, Qt.AlignCenter)
        vColumn3.addWidget(self.availableUserTable)
        grid.addLayout(vColumn3, 2, 2)



        grid.addWidget(saveUsersBtn, 4, 0, 1, 3, Qt.AlignCenter)

        self.manageUsersTabWidget = QWidget()
        self.manageUsersTabWidget.setLayout(grid)
    
    # Populates the dropdown with available proposals
    def updateDropdown(self):
        self.proposalList = self.experimentPropApi.listBeamlineProposals()
        self.userDropdown.addItem('All Users', -1)
        for proposal in self.proposalList:
            fm = QFontMetrics(QFont("times", 12))
            elidedText = fm.elidedText(proposal['title'], Qt.ElideRight, 250)
            self.userDropdown.addItem(elidedText, proposal['id'])


    # Expands the given row to fit the size of its content
    def expandRow(self, row, column, table):
        table.resizeRowToContents(row)

    # Populates both user tables with their corresponding users
    def updateUsers(self):
        # Setup the available users table
        id = self.userDropdown.itemData(self.userDropdown.currentIndex(), Qt.UserRole)
        if id != -1:
            proposal = self.experimentPropApi.getBeamlineProposal(id)
            self.availableUserTable.model().setRowCount(len(proposal['experimenters']))
            allUsers = proposal['experimenters']
        else:
            allUsers = self.userApi.getUsers()
            self.availableUserTable.model().setRowCount(len(allUsers))
        objectAllUsers = []
        for x in allUsers:
            try:
                objectAllUsers.append(UserInfo(x['badge'], x['firstName'], x['id'],  x['lastName'], x['email']))
            except KeyError, key:
                if key.message == 'badge':
                    continue
                else:
                    objectAllUsers.append(UserInfo(x['badge'], x['firstName'], x['id'],  x['lastName'], ''))
        self.availableUserTable.model().setColumnCount(4)
        self.availableUserTable.horizontalHeader().setStretchLastSection(True)
        self.availableUserTable.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.availableUserTable.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.availableUserTable.model().setHorizontalHeaderLabels('Badge;First;Last;Email;'.split(';'))
        self.availableUserTable.doubleClicked.connect(lambda row, column: self.expandRow(row, column, self.availableUserTable))

        i = 0
        currentIDs = [x.getID() for x in self.parent.currentUsers]
        for experimenter in objectAllUsers:
            if experimenter.getID() in currentIDs:
                self.availableUserTable.model().removeRow(i)
                continue
            rowBadge = QStandardItem(experimenter.getBadge())
            if rowBadge is None:
                continue
            rowBadge.setData(Qt.UserRole, experimenter.getID())
            rowFirstName = QStandardItem(experimenter.getFirstName())
            rowLastName = QStandardItem(experimenter.getLastName())
            rowEmail = QStandardItem(experimenter.getEmail())
            if rowEmail is None:
                rowEmail = QStandardItem('')
            self.availableUserTable.model().setItem(i, 0, rowBadge)
            self.availableUserTable.model().setItem(i, 1, rowFirstName)
            self.availableUserTable.model().setItem(i, 2, rowLastName)
            self.availableUserTable.model().setItem(i, 3, rowEmail)
            if (i % 2) == 0:
                rowBadge.setBackground(QColor.fromRgb(230, 241, 245))
                rowFirstName.setBackground(QColor.fromRgb(230, 241, 245))
                rowLastName.setBackground(QColor.fromRgb(230, 241, 245))
                rowEmail.setBackground(QColor.fromRgb(230, 241, 245))
            i += 1

        # Set up the current users table
        self.currentUserTable.setRowCount(len(self.parent.currentUsers))
        self.currentUserTable.setColumnCount(4)
        self.currentUserTable.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.currentUserTable.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.currentUserTable.setHorizontalHeaderLabels('Badge;First;Last;Email;'.split(';'))
        self.currentUserTable.cellDoubleClicked.connect(lambda row, column: self.expandRow(row, column, self.currentUserTable))

        i = 0
        for experimenter in self.parent.currentUsers:
            rowBadge = QTableWidgetItem(experimenter.getBadge())
            rowBadge.setData(Qt.UserRole, experimenter.getID())
            rowFirstName = QTableWidgetItem(experimenter.getFirstName())
            rowLastName = QTableWidgetItem(experimenter.getLastName())
            try:
                rowEmail = QTableWidgetItem(experimenter.getEmail())
            except KeyError:
                rowEmail = QTableWidgetItem('')
            self.currentUserTable.setItem(i, 0, rowBadge)
            self.currentUserTable.setItem(i, 1, rowFirstName)
            self.currentUserTable.setItem(i, 2, rowLastName)
            self.currentUserTable.setItem(i, 3, rowEmail)
            if (i % 2) == 0:
                rowBadge.setBackground(QColor.fromRgb(230, 241, 245))
                rowFirstName.setBackground(QColor.fromRgb(230, 241, 245))
                rowLastName.setBackground(QColor.fromRgb(230, 241, 245))
                rowEmail.setBackground(QColor.fromRgb(230, 241, 245))
            i += 1

        self.currentUserTable.horizontalHeader().setStretchLastSection(True)

    # Moves users from one table to the other
    def moveUsers(self):
        for selection in self.availableUserTable.model().selectedItems():
            userID = self.availableUserTable.model().item(selection.row(), 0).data(Qt.UserRole)

        selectedAvailable = [self.availableUserTable.model().item(x.row(), 0).data(Qt.UserRole) for x in self.availableUserTable.model().selectedItems()]
        currentAvailable = [self.currentUserTable.item(x.row(), 0).data(Qt.UserRole) for x in self.currentUserTable.selectedItems()]

        for id in selectedAvailable:
            self.parent.currentUsers.append(self.userApi.getUserById(id))

        for id in currentAvailable:
            self.parent.currentUsers.remove()
          #self.parent.currentUsers.remove(

        #selectedAvailable = [x.row(0).text() for x in self.availableUserTable.selectedItems()]
        #currentAvailable = self.currentUserTable.item(self.currentUserTable.currentRow(), 0)

    def addFilter(self):
        self.updateUsers
