#!/usr/bin/env python

from dm.cat_web_service.api.datasetCatApi import DatasetCatApi
from dm.common.exceptions.invalidRequest import InvalidRequest
from catWebServiceSessionCli import CatWebServiceSessionCli

class GetExperimentDatasetCli(CatWebServiceSessionCli):
    def __init__(self):
        CatWebServiceSessionCli.__init__(self, validArgCount=self.ANY_NUMBER_OF_POSITIONAL_ARGS)
        self.addOption('', '--dataset', dest='datasetName', help='Dataset name.')
        self.addOption('', '--experiment', dest='experimentName', help='Experiment name.')

    def checkArgs(self):
        if self.options.experimentName is None:
            raise InvalidRequest('Experiment name must be provided.')
        if self.options.datasetName is None:
            raise InvalidRequest('Dataset name must be provided.')

    def getExperimentName(self):
        return self.options.experimentName

    def getDatasetName(self):
        return self.options.datasetName

    def runCommand(self):
        self.parseArgs(usage="""
    dm-get-experiment-dataset --dataset=DATASETNAME --experiment=EXPERIMENTNAME 

Description:
    Retrieve experiment dataset metadata from the catalog. 
        """)
        self.checkArgs()
        api = DatasetCatApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        datasetMetadata = api.getExperimentDataset(self.getExperimentName(), self.getDatasetName())
        print datasetMetadata.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = GetExperimentDatasetCli()
    cli.run()

