#!/usr/bin/env python

from dm.common.exceptions.invalidRequest import InvalidRequest
from dm.ds_web_service.api.userDsApi import UserDsApi
from dsWebServiceSessionCli import DsWebServiceSessionCli

class AddUserSystemRoleCli(DsWebServiceSessionCli):
    def __init__(self):
        DsWebServiceSessionCli.__init__(self)
        self.addOption('', '--username', dest='username', help='User username.')
        self.addOption('', '--role', dest='roleName', help='Role name can be either \'Administrator\' or \'Manager\'.')
        self.addOption('', '--station', dest='experimentStationName', help='Experiment station name (should not be provided for administrator system role).')

    def checkArgs(self):
        if not self.options.username:
            raise InvalidRequest('Username must be provided.')
        if not self.options.roleName:
            raise InvalidRequest('Role name must be provided.')

    def getUsername(self):
        return self.options.username

    def getExperimentStationName(self):
        return self.options.experimentStationName

    def getRoleName(self):
        return self.options.roleName

    def runCommand(self):
        self.parseArgs(usage="""
    dm-add-user-experiment-role --username=USERNAME 
        --role=ROLENAME
        [--station=EXPERIMENTSTATIONNAME]

Description:
    Assigns system role to the given user. The 'Administrator' role cannot be accompanied with experiment station, while the 'Manager' role requires it.
        """)
        self.checkArgs()
        api = UserDsApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        userSystemRole = api.addUserSystemRole(self.getUsername(), self.getRoleName(), self.getExperimentStationName())
        print userSystemRole.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = AddUserSystemRoleCli()
    cli.run()

